<?xml version='1.0' encoding='ISO-8859-1'?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:myxhtml="http://www.w3.org/1999/xhtml"  exclude-result-prefixes="myxhtml">
<!--	****************** LICENSE *****************************************************
	******************************************************************************
	Semark.xsl : Semantic markup using XSL transformation
	
	author:  Xavier Boully
	web:     http://www.ultra-fluide.com/
	mail : xavier.boully@ultra-fluide.com
	date : july 2004
	please note: we are interested in receiving information about the specific use done of this code and the 
	context in which it is used.
	
	Copyright (C) 2004 Xavier Boully
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details:
	- http://www.gnu.org/copyleft/gpl.html
	or write to :
	-  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
	
	****************** VERSION = Semark.xsl 1.0.0 **************************************
	******************************************************************************
	
	****************** COMPATIBILITY ***********************************************
	******************************************************************************
	This XSL transformation is compatible with W3C's XSL 1.0 recommendation and uses no extensions 
	in order to maintain optimum compatibility (this makes the code a little heavy at times).
	It functions with all the XSLT processors we tested:
	Saxon, Xalan, Libxslt (via php), as well as processors embedded in Mozilla and IE6 browsers.
	
	****************** OPERATING OPTIONS *****************************************
	******************************************************************************
	This file is an XSLT "style sheet" which makes it possible to automate the semantic markup 
	of a document with XML content (for example a XHTML web page) using vocabulary defined in an external XHTML lexicon.
	This XSLT transformation allows for the following parameters:
	- lexicon-name: string designating the name of the file which contains the lexicon to be applied, value by default: 'lexicon.htm'
	- to-parse: string designating the list of elements to be parsed. value by default: 'body'
	- not-to-parse: string designating the list of elements to be excluded from parsing. value by default: empty string
	- marker-name: string designating the tag and attribute used for the markup. value by default: 'acronym title'
	- max-per-doc: number indicating the maximum number of markups in the document for each entry in the lexicon
	                       value by default: -1 (no limit)
	- element-to-limit: string designating the list of elements for which we need to set a limit for the number of markups per entry in the lexicon.
				value by default: empty string	       
	- max-per-element: number indicating the maximum number of markups per element designated in the element-to-limit parameter
	                            value by default: -1 (no limit)
-->
<xsl:output method="xml" omit-xml-declaration="yes" encoding="ISO-8859-1" doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN" doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd" />
<!--external parameters-->
<xsl:param name="lexique-name"/> <!-- string for the path to the lexicon -->
<xsl:param name="to-parse" /> <!-- string designating the list of elements to be parsed-->
<xsl:param name="not-to-parse" /> <!-- string designating the list of elements to be excluded from parsing -->
<xsl:param name="marker-name" /> <!-- string designating the tag and attribute -->
<xsl:param name="max-per-doc" /> <!-- maximum number of markups per expression in the document, -1 means no limit-->
<xsl:param name="max-per-element" /> <!-- maximum number of markups per expression for the limited elements -->
<xsl:param name="element-to-limit" /> <!-- string designating the list of elements affected by a markup limit -->
<!--END external parameters-->
<!--verification and default values for the external parameters-->
<xsl:param name="lexique-uri"> <!--we use the lexicon name provided by the external parameter, or by default 'lexicon.htm' -->
	<xsl:choose>
		<xsl:when test="$lexique-name = ''">
			<xsl:value-of select="'lexicon.html'"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$lexique-name"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="parsed-tag"> <!--list of tags to be parsed, by default element body -->
	<xsl:variable name="temp" select="normalize-space($to-parse)"/>
	<xsl:choose>
		<xsl:when test="$temp = ''"><xsl:value-of select="' body '"/></xsl:when>
		<xsl:otherwise><xsl:value-of select="concat(' ',$temp,' ')"/></xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="excluded-tag" select="concat(' ',normalize-space($not-to-parse),' ')"/> <!--list of tags to be excluded, by default we exclude none -->
<xsl:param name="marker"> <!-- tag to be used for markup, acronym by default -->
	<xsl:variable name="temp" select="substring-before(normalize-space($marker-name),' ')"/>
	<xsl:choose>
		<xsl:when test="$temp = ''"><xsl:value-of select="'acronym'"/></xsl:when>
		<xsl:otherwise><xsl:value-of select="$temp"/></xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="marker-attribute"> <!-- attribute to be used for the markup tag, title by default -->
	<xsl:variable name="temp" select="substring-after(normalize-space($marker-name),' ')"/>
	<xsl:choose>
		<xsl:when test="$temp = ''"><xsl:value-of select="'title'"/></xsl:when>
		<xsl:otherwise><xsl:value-of select="$temp"/></xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="limit-in-doc"> <!-- maximum number of markups per expression in the document, no maximum by default (-1) -->
	<xsl:variable name="temp" select="normalize-space($max-per-doc)"/>
	<xsl:choose>
		<xsl:when test="$temp = ''">
			<xsl:value-of select="-1"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="number($temp)"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="limit-in-elements"> <!-- maximum number of markups per expression for the limited elements, no maximum by default (-1) -->
	<xsl:variable name="temp" select="normalize-space($max-per-element)"/>
	<xsl:choose>
		<xsl:when test="$temp = ''">
			<xsl:value-of select="-1"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="number($temp)"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="limited-elements" select="concat(' ',normalize-space($element-to-limit),' ')"/> <!-- elements for which we are imposing a limit on the number of markups, empty by default -->
<!--END verification and default values for the external parameters-->
<!-- Initialization of the variables -->
<xsl:param name="lexique" select="document($lexique-uri)//myxhtml:dl/myxhtml:dt"/>
<xsl:param name="taille-lexique" select="count($lexique)"/>
<xsl:param name="caracteres" select="'abcdefghijklmnopqrstuvwxyz'"/>
<xsl:param name="caracteres-normalises" select="'AABCCDEEEEFGHIIJKLMNOOPQRSTUUVWXYZ'"/>
<xsl:param name="initial-max">
	<xsl:call-template name="init-vecteur-max" >
		<xsl:with-param name="indice" select="0"/>
		<xsl:with-param name="vecteur" select="'#'"/>	
		<xsl:with-param name="value" select="$limit-in-doc"/>			
	</xsl:call-template>
</xsl:param>
<xsl:param name="vector-0">
	<xsl:call-template name="init-vecteur-max" >
		<xsl:with-param name="indice" select="0"/>
		<xsl:with-param name="vecteur" select="'#'"/>	
		<xsl:with-param name="value" select="0"/>			
	</xsl:call-template>
</xsl:param>
<xsl:template name="init-vecteur-max">
<xsl:param name="indice"/>
<xsl:param name="vecteur"/>
<xsl:param name="value"/>
	<xsl:choose>
		<xsl:when test="$indice &lt; $taille-lexique">
			<xsl:call-template name="init-vecteur-max">
				<xsl:with-param name="indice" select="$indice + 1"/>
				<xsl:with-param name="vecteur" select="concat($vecteur,$indice + 1,'.',$value,'#')"/>
				<xsl:with-param name="value" select="$value"/>
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise><xsl:value-of select="$vecteur"/></xsl:otherwise>
	</xsl:choose>
</xsl:template>
<!-- END Initialization of the variables -->

<xsl:template match="/">
	<xsl:apply-templates select="(*| comment())[1]">
		<xsl:with-param name="parent-max" select="$initial-max"/>
		<xsl:with-param name="parent-parse" select="'NA'"/>
	</xsl:apply-templates>
</xsl:template>

<xsl:template match="@*|*|comment() | text()">
<xsl:param name="parent-max"/>
<xsl:param name="parent-parse"/>
<xsl:param name="current-max">
	<xsl:choose>
		<xsl:when test="$limit-in-elements != -1 and contains($limited-elements,concat(' ',normalize-space(name()),' ')) and name() != '' ">
			<xsl:call-template name="compute-max-with-number" >
				<xsl:with-param name="input" select="substring($parent-max,2)"/>
				<xsl:with-param name="output" select="''"/>		
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise><xsl:value-of select="$parent-max"/></xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="current-parse">
	<xsl:choose>
		<xsl:when test="($parent-parse = 'NO') or (contains($excluded-tag,concat(' ',normalize-space(name()),' ')) and name() != '')">
			<xsl:value-of select="'NO'"/>
		</xsl:when>
		<xsl:when test="($parent-parse = 'YES') or (contains($parsed-tag,concat(' ',normalize-space(name()),' ')) and name() != '')">
			<xsl:value-of select="'YES'"/>
		</xsl:when>
		<xsl:otherwise><xsl:value-of select="'NA'"/></xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="current-node-marker-count-analyse">
	<xsl:choose>	
		<xsl:when test="$parent-parse = 'YES' and $taille-lexique != 0 and self::text() ">
				<xsl:call-template name="analyse-count">
					<xsl:with-param name="position-dt" select="1"/>
					<xsl:with-param name="marker-vector" select="'#'"/>
					<xsl:with-param name="contenu" select="translate(.,$caracteres,$caracteres-normalises)"/>
				</xsl:call-template>
		</xsl:when>
		<xsl:otherwise>
				<xsl:apply-templates select="(*|text())[1] " mode="count">
					<xsl:with-param name="parent-parse" select="$current-parse"/>
					<xsl:with-param name="parent-max" select="$current-max"/>
				</xsl:apply-templates>
		</xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="current-node-marker-count">
	<xsl:choose>	
		<xsl:when test="$current-node-marker-count-analyse = '' "><xsl:value-of select="$vector-0"/></xsl:when>
		<xsl:otherwise><xsl:value-of select="$current-node-marker-count-analyse"/></xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="reduce-parent-max">
	<xsl:call-template name="reduce-max-with-vector" >
		<xsl:with-param name="input-to-reduce" select="substring($parent-max,2)"/>
		<xsl:with-param name="reducer" select="substring($current-node-marker-count,2)"/>
		<xsl:with-param name="output" select="''"/>		
	</xsl:call-template>
</xsl:param>
<xsl:choose>	
	<xsl:when test="$parent-parse = 'YES' and $taille-lexique != 0 and self::text() ">
		<xsl:call-template name="analyse" >
			<xsl:with-param name="contenu" select="."/>
			<xsl:with-param name="position-dt" select="1"/>
			<xsl:with-param name="parent-max" select="$current-max"/>
		</xsl:call-template>
	</xsl:when>
	<xsl:otherwise>
		<xsl:copy>
			<xsl:apply-templates select="node()[1] | @*">
				<xsl:with-param name="parent-parse" select="$current-parse"/>
				<xsl:with-param name="parent-max" select="$current-max"/>
			</xsl:apply-templates>
		</xsl:copy>
	</xsl:otherwise>
</xsl:choose>
	<xsl:apply-templates select="(following-sibling::*|following-sibling::comment()|following-sibling::text())[1]">
		<xsl:with-param name="parent-parse" select="$parent-parse"/>
		<xsl:with-param name="parent-max" select="$reduce-parent-max"/>
	</xsl:apply-templates>
</xsl:template>

<xsl:template match="*|text()" mode="count">
<xsl:param name="parent-max"/>
<xsl:param name="parent-parse"/>
<xsl:param name="current-max">
	<xsl:choose>
		<xsl:when test="$limit-in-elements != -1 and contains($limited-elements,concat(' ',normalize-space(name()),' ')) and name() != '' ">
			<xsl:call-template name="compute-max-with-number" >
				<xsl:with-param name="input" select="substring($parent-max,2)"/>
				<xsl:with-param name="output" select="''"/>		
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise><xsl:value-of select="$parent-max"/></xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="current-parse">
	<xsl:choose>
		<xsl:when test="($parent-parse = 'NO') or (contains($excluded-tag,concat(' ',normalize-space(name()),' ')) and name() != '')">
			<xsl:value-of select="'NO'"/>
		</xsl:when>
		<xsl:when test="($parent-parse = 'YES') or (contains($parsed-tag,concat(' ',normalize-space(name()),' ')) and name() != '')">
			<xsl:value-of select="'YES'"/>
		</xsl:when>
		<xsl:otherwise><xsl:value-of select="'NA'"/></xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="current-node-marker-count-analyse">
	<xsl:choose>	
		<xsl:when test="$parent-parse = 'YES' and $taille-lexique != 0 and self::text() ">
			<xsl:variable name="marker-count-potential">
				<xsl:call-template name="analyse-count">
					<xsl:with-param name="position-dt" select="1"/>
					<xsl:with-param name="marker-vector" select="'#'"/>
					<xsl:with-param name="contenu" select="translate(.,$caracteres,$caracteres-normalises)"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:call-template name="cap-max-with-vector" >
				<xsl:with-param name="input-to-cap" select="substring($marker-count-potential,2)"/>
				<xsl:with-param name="cap" select="substring($parent-max,2)"/>
				<xsl:with-param name="output" select="''"/>		
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise>
			<xsl:apply-templates select="(*|text())[1] " mode="count">
				<xsl:with-param name="parent-parse" select="$current-parse"/>
				<xsl:with-param name="parent-max" select="$current-max"/>
			</xsl:apply-templates>
		</xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="current-node-marker-count">
	<xsl:choose>	
		<xsl:when test="$current-node-marker-count-analyse = '' "><xsl:value-of select="$vector-0"/></xsl:when>
		<xsl:otherwise><xsl:value-of select="$current-node-marker-count-analyse"/></xsl:otherwise>
	</xsl:choose>
</xsl:param>
<xsl:param name="reduce-parent-max">
	<xsl:call-template name="reduce-max-with-vector" >
		<xsl:with-param name="input-to-reduce" select="substring($parent-max,2)"/>
		<xsl:with-param name="reducer" select="substring($current-node-marker-count,2)"/>
		<xsl:with-param name="output" select="''"/>		
	</xsl:call-template>
</xsl:param>
<xsl:param name="next-node-marker-count">
	<xsl:apply-templates select="(following-sibling::*|following-sibling::text())[1]" mode="count">
		<xsl:with-param name="parent-parse" select="$parent-parse"/>
		<xsl:with-param name="parent-max" select="$reduce-parent-max"/>
	</xsl:apply-templates>
</xsl:param>
<xsl:param name="sum">
<xsl:call-template name="add-vectors" >
			<xsl:with-param name="input-1" select="substring($current-node-marker-count,2)"/>
			<xsl:with-param name="input-2" select="substring($next-node-marker-count,2)"/>
			<xsl:with-param name="output" select="''"/>		
		</xsl:call-template>
</xsl:param>
<xsl:choose>	
	<xsl:when test="$next-node-marker-count = '' ">
		<xsl:value-of select="$current-node-marker-count"/>
	</xsl:when>
	<xsl:otherwise>
		<xsl:call-template name="add-vectors" >
			<xsl:with-param name="input-1" select="substring($current-node-marker-count,2)"/>
			<xsl:with-param name="input-2" select="substring($next-node-marker-count,2)"/>
			<xsl:with-param name="output" select="''"/>		
		</xsl:call-template>
	</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="analyse-count">
<xsl:param name="marker-vector"/>
<xsl:param name="position-dt"/>
<xsl:param name="contenu"/>
<xsl:variable name="entree" select="translate(normalize-space($lexique[$position-dt]),$caracteres,$caracteres-normalises)"/>
<xsl:choose>
	<xsl:when test="$position-dt &gt; $taille-lexique">
		<xsl:value-of select="$marker-vector"/>
	</xsl:when>
	<xsl:otherwise>
		<xsl:call-template name="analyse-count" >
			<xsl:with-param name="contenu" select="$contenu"/>
			<xsl:with-param name="position-dt" select="$position-dt + 1"/>
			<xsl:with-param name="marker-vector">
				<xsl:variable name="dt-count">
					<xsl:call-template name="dt-count" >
						<xsl:with-param name="contenu" select="$contenu"/>
						<xsl:with-param name="dt-count" select="0"/>
						<xsl:with-param name="entree" select="$entree"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:value-of select="concat($marker-vector,$position-dt,'.',$dt-count,'#')"/>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="dt-count">
<xsl:param name="dt-count"/>
<xsl:param name="entree"/>
<xsl:param name="contenu"/>
	<xsl:choose>
		<xsl:when test="contains($contenu,$entree)">
			<xsl:variable name="debut-contenu" select="substring-before($contenu,$entree)"/>
			<xsl:variable name="fin-contenu" select="substring-after($contenu,$entree)"/>
			<xsl:variable name="sep-avant" select="substring($debut-contenu,string-length($debut-contenu))"/>
			<xsl:variable name="sep-apres" select="substring($fin-contenu,1,1)"/>
			<xsl:variable name="dt-count-increment">
				<xsl:choose>
					<xsl:when test="($sep-avant='' or $sep-avant=' ' or $sep-avant=',' or $sep-avant='=' or $sep-avant='(' or $sep-avant='-') and ($sep-apres='' or $sep-apres=' ' or $sep-apres=',' or $sep-apres='.' or $sep-apres='=' or $sep-apres=')' or $sep-apres='-')">
						<xsl:value-of select="1"/>
					</xsl:when>
					<xsl:otherwise><xsl:value-of select="0"/></xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:call-template name="dt-count" >
				<xsl:with-param name="contenu" select="$fin-contenu"/>
				<xsl:with-param name="entree" select="$entree"/>
				<xsl:with-param name="dt-count" select="$dt-count + $dt-count-increment"/>
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise><xsl:value-of select="$dt-count"/></xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="analyse">
<xsl:param name="parent-max"/>
<xsl:param name="contenu" />
<xsl:param name="position-dt" />
<xsl:variable name="entree" select="translate(normalize-space($lexique[$position-dt]),$caracteres,$caracteres-normalises)"/>
<xsl:variable name="contenu-normalise" select="translate($contenu,$caracteres,$caracteres-normalises)"/>
	<xsl:choose>
		<xsl:when test="contains($contenu-normalise,$entree) and substring-before(substring-after($parent-max,concat('#',$position-dt,'.')),'#') != 0">
			<xsl:variable name="debut-contenu" select="substring($contenu,1,string-length(substring-before($contenu-normalise,$entree)))"/>
			<xsl:variable name="milieu-contenu" select="substring($contenu,string-length($debut-contenu)+1,string-length($entree))"/>
			<xsl:variable name="fin-contenu" select="substring($contenu,string-length($debut-contenu)+string-length($entree)+1)"/>
			<xsl:variable name="sep-avant" select="substring($debut-contenu,string-length($debut-contenu))"/>
			<xsl:variable name="sep-apres" select="substring($fin-contenu,1,1)"/>
			<xsl:choose>
				<xsl:when test="$position-dt &lt; $taille-lexique">
					<xsl:call-template name="analyse" >
						<xsl:with-param name="contenu" select="$debut-contenu"/>
						<xsl:with-param name="position-dt" select="$position-dt + 1"/>
						<xsl:with-param name="parent-max" select="$parent-max"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$debut-contenu"/>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:variable name="definition" select="$lexique[$position-dt]/following-sibling::myxhtml:dd"/>
			<xsl:choose>
				<xsl:when test="($sep-avant='' or $sep-avant=' ' or $sep-avant=',' or $sep-avant='=' or $sep-avant='(' or $sep-avant='-') and ($sep-apres='' or $sep-apres=' ' or $sep-apres=',' or $sep-apres='.' or $sep-apres='=' or $sep-apres=')' or $sep-apres='-')">
					 <xsl:element name="{$marker}" >    
					     <xsl:attribute name="{$marker-attribute}">
						     <xsl:value-of select="$definition"/>
					      </xsl:attribute>
					      <xsl:value-of select="$milieu-contenu"/>
					</xsl:element>
					<xsl:call-template name="analyse" >
						<xsl:with-param name="contenu" select="$fin-contenu"/>
						<xsl:with-param name="position-dt" select="$position-dt"/>
						<xsl:with-param name="parent-max">
							<xsl:choose>
								<xsl:when test="substring-before(substring-after($parent-max,concat('#',$position-dt,'.')),'#') = -1">
									<xsl:value-of select="$parent-max"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:variable name="string-end" select="substring-after($parent-max,concat('#',$position-dt,'.'))"/>
									<xsl:value-of select="concat(substring-before($parent-max,$string-end) , string(number(substring-before($string-end,'#')) - 1) , '#' , substring-after($string-end,'#') )"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$milieu-contenu"/>
					<xsl:call-template name="analyse" >
						<xsl:with-param name="contenu" select="$fin-contenu"/>
						<xsl:with-param name="position-dt" select="$position-dt"/>
						<xsl:with-param name="parent-max" select="$parent-max"/>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:otherwise>
			<xsl:choose>
				<xsl:when test="$position-dt = $taille-lexique">
					<xsl:value-of select="$contenu"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="analyse" >
						<xsl:with-param name="contenu" select="$contenu"/>
						<xsl:with-param name="position-dt" select="$position-dt + 1"/>
						<xsl:with-param name="parent-max" select="$parent-max"/>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="compute-max-with-number">
<xsl:param name="input"/>
<xsl:param name="output"/>
<xsl:choose>
	<xsl:when test="contains($input,'.')">
		<xsl:variable name="initial-block" select="substring-before($input,'#')"/>
		<xsl:variable name="new-max">
			<xsl:choose>
				<xsl:when test="(number(substring-after($initial-block,'.')) &gt; $limit-in-elements) or number(substring-after($initial-block,'.')) = -1">
					<xsl:value-of select="$limit-in-elements"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="number(substring-after($initial-block,'.'))"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:call-template name="compute-max-with-number">
			<xsl:with-param name="input" select="substring-after($input,'#')"/>
			<xsl:with-param name="output" select="concat($output,'#',substring-before($initial-block,'.'),'.',$new-max)"/>		
		</xsl:call-template>	
	</xsl:when>
	<xsl:otherwise>
		<xsl:value-of select="concat($output,'#')"/>
	</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="reduce-max-with-vector">
<xsl:param name="input-to-reduce"/>
<xsl:param name="reducer"/>
<xsl:param name="output"/>
<xsl:choose>
	<xsl:when test="contains($input-to-reduce,'.')">
		<xsl:variable name="initial-block" select="substring-before($input-to-reduce,'#')"/>
		<xsl:variable name="reducer-block" select="substring-before($reducer,'#')"/>
		<xsl:variable name="new-max">
			<xsl:choose>
				<xsl:when test="number(substring-after($initial-block,'.')) = -1">
					<xsl:value-of select="number(substring-after($initial-block,'.'))"/>
				</xsl:when>
				<xsl:when test="number(substring-after($initial-block,'.')) &gt; number(substring-after($reducer-block,'.'))">
					<xsl:value-of select="number(substring-after($initial-block,'.')) - number(substring-after($reducer-block,'.'))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="0"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:call-template name="reduce-max-with-vector">
			<xsl:with-param name="input-to-reduce" select="substring-after($input-to-reduce,'#')"/>
			<xsl:with-param name="reducer" select="substring-after($reducer,'#')"/>
			<xsl:with-param name="output" select="concat($output,'#',substring-before($initial-block,'.'),'.',$new-max)"/>		
		</xsl:call-template>	
	</xsl:when>
	<xsl:otherwise>
		<xsl:value-of select="concat($output,'#')"/>
	</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="cap-max-with-vector">
<xsl:param name="input-to-cap"/>
<xsl:param name="cap"/>
<xsl:param name="output"/>
<xsl:choose>
	<xsl:when test="contains($input-to-cap,'.')">
		<xsl:variable name="initial-block" select="substring-before($input-to-cap,'#')"/>
		<xsl:variable name="cap-block" select="substring-before($cap,'#')"/>
		<xsl:variable name="new-max">
			<xsl:choose>
				<xsl:when test="number(substring-after($cap-block,'.')) = -1">
					<xsl:value-of select="number(substring-after($initial-block,'.'))"/>
				</xsl:when>
				<xsl:when test="number(substring-after($initial-block,'.')) &gt; number(substring-after($cap-block,'.'))">
					<xsl:value-of select="number(substring-after($cap-block,'.'))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="number(substring-after($initial-block,'.'))"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:call-template name="cap-max-with-vector">
			<xsl:with-param name="input-to-cap" select="substring-after($input-to-cap,'#')"/>
			<xsl:with-param name="cap" select="substring-after($cap,'#')"/>
			<xsl:with-param name="output" select="concat($output,'#',substring-before($initial-block,'.'),'.',$new-max)"/>		
		</xsl:call-template>	
	</xsl:when>
	<xsl:otherwise>
		<xsl:value-of select="concat($output,'#')"/>
	</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="add-vectors">
<xsl:param name="input-1"/>
<xsl:param name="input-2"/>
<xsl:param name="output"/>
<xsl:choose>
	<xsl:when test="contains($input-1,'.')">
		<xsl:variable name="block-1" select="substring-before($input-1,'#')"/>
		<xsl:variable name="block-2" select="substring-before($input-2,'#')"/>
		<xsl:call-template name="add-vectors">
			<xsl:with-param name="input-1" select="substring-after($input-1,'#')"/>
			<xsl:with-param name="input-2" select="substring-after($input-2,'#')"/>
			<xsl:with-param name="output" select="concat($output,'#',substring-before($block-1,'.'),'.',number(substring-after($block-1,'.')) + number(substring-after($block-2,'.')))"/>		
		</xsl:call-template>	
	</xsl:when>
	<xsl:otherwise>
		<xsl:value-of select="concat($output,'#')"/>
	</xsl:otherwise>
</xsl:choose>
</xsl:template>

</xsl:stylesheet>